@@ -7,7 +7,7 @@ bool Archive::GetComment(Array<wchar> *CmtData)
   SaveFilePos SavePos(*this);
 
 #ifndef SFX_MODULE
-  ushort CmtLength;
+  uint CmtLength;
   if (Format==RARFMT14)
   {
     Seek(SFXSize+SIZEOF_MAINHEAD14,SEEK_SET);
@@ -45,14 +45,14 @@ bool Archive::GetComment(Array<wchar> *CmtData)
   if (Format==RARFMT14 && MainHead.PackComment || Format!=RARFMT14 && CommHead.Method!=0x30)
   {
     if (Format!=RARFMT14 && (CommHead.UnpVer < 15 || CommHead.UnpVer > VER_UNPACK || CommHead.Method > 0x35))
-      return(false);
+      return false;
     ComprDataIO DataIO;
     DataIO.SetTestMode(true);
     uint UnpCmtLength;
     if (Format==RARFMT14)
     {
 #ifdef RAR_NOCRYPT
-      return(false);
+      return false;
 #else
       UnpCmtLength=GetByte();
       UnpCmtLength+=(GetByte()<<8);
@@ -84,16 +84,20 @@ bool Archive::GetComment(Array<wchar> *CmtData)
       size_t UnpDataSize;
       DataIO.GetUnpackedData(&UnpData,&UnpDataSize);
 #ifdef _WIN_ALL
+      // If we ever decide to extend it to Android, we'll need to alloc
+      // 4x memory for OEM to UTF-8 output here.
       OemToCharBuffA((char *)UnpData,(char *)UnpData,(DWORD)UnpDataSize);
 #endif
       CmtData->Alloc(UnpDataSize+1);
       memset(CmtData->Addr(0),0,CmtData->Size()*sizeof(wchar));
-      CharToWide((char *)UnpData,CmtData->Addr(0),UnpDataSize);
+      CharToWide((char *)UnpData,CmtData->Addr(0),CmtData->Size());
       CmtData->Alloc(wcslen(CmtData->Addr(0)));
     }
   }
   else
   {
+    if (CmtLength==0)
+      return false;
     Array<byte> CmtRaw(CmtLength);
     Read(&CmtRaw[0],CmtLength);
 
@@ -105,6 +109,8 @@ bool Archive::GetComment(Array<wchar> *CmtData)
     CmtData->Alloc(CmtLength+1);
     CmtRaw.Push(0);
 #ifdef _WIN_ALL
+    // If we ever decide to extend it to Android, we'll need to alloc
+    // 4x memory for OEM to UTF-8 output here.
     OemToCharA((char *)&CmtRaw[0],(char *)&CmtRaw[0]);
 #endif
     CharToWide((char *)&CmtRaw[0],CmtData->Addr(0),CmtLength);
@@ -143,11 +149,10 @@ bool Archive::ReadCommentData(Array<wchar> *CmtData)
 
 void Archive::ViewComment()
 {
-#ifndef GUI
   if (Cmd->DisableComment)
     return;
   Array<wchar> CmtBuf;
-  if (GetComment(&CmtBuf))
+  if (GetComment(&CmtBuf)) // In GUI too, so "Test" command detects broken comments.
   {
     size_t CmtSize=CmtBuf.Size();
     wchar *ChPtr=wcschr(&CmtBuf[0],0x1A);
@@ -156,7 +161,6 @@ void Archive::ViewComment()
     mprintf(L"\n");
     OutComment(&CmtBuf[0],CmtSize);
   }
-#endif
 }
 
 
