@@ -53,9 +53,11 @@ size_t Archive::SearchBlock(HEADER_TYPE HeaderType)
 
 size_t Archive::SearchSubBlock(const wchar *Type)
 {
-  size_t Size;
+  size_t Size,Count=0;
   while ((Size=ReadHeader())!=0 && GetHeaderType()!=HEAD_ENDARC)
   {
+    if ((++Count & 127)==0)
+      Wait();
     if (GetHeaderType()==HEAD_SERVICE && SubHead.CmpName(Type))
       return Size;
     SeekToNext();
@@ -306,21 +308,20 @@ size_t Archive::ReadHeader15()
 
         if (FileBlock)
         {
+          *hd->FileName=0;
           if ((hd->Flags & LHD_UNICODE)!=0)
           {
             EncodeFileName NameCoder;
             size_t Length=strlen(FileName);
             Length++;
-            NameCoder.Decode(FileName,(byte *)FileName+Length,
-                             NameSize-Length,hd->FileName,
-                             ASIZE(hd->FileName));
+            if (ReadNameSize>Length)
+              NameCoder.Decode(FileName,(byte *)FileName+Length,
+                               ReadNameSize-Length,hd->FileName,
+                               ASIZE(hd->FileName));
           }
-          else
-            *hd->FileName=0;
 
-          char AnsiName[NM];
-          IntToExt(FileName,AnsiName,ASIZE(AnsiName));
-          GetWideName(AnsiName,hd->FileName,hd->FileName,ASIZE(hd->FileName));
+          if (*hd->FileName==0)
+            ArcCharToWide(FileName,hd->FileName,ASIZE(hd->FileName),ACTW_OEM);
 
 #ifndef SFX_MODULE
           ConvertNameCase(hd->FileName);
@@ -391,6 +392,8 @@ size_t Archive::ReadHeader15()
               byte CurByte=Raw.Get1();
               rlt.Reminder|=(((uint)CurByte)<<((J+3-count)*8));
             }
+            // Convert from 100ns RAR precision to REMINDER_PRECISION.
+            rlt.Reminder*=RarTime::REMINDER_PRECISION/10000000;
             CurTime->SetLocal(&rlt);
           }
         }
@@ -561,7 +564,7 @@ size_t Archive::ReadHeader50()
 
   if (Decrypt)
   {
-#if defined(SHELL_EXT) || defined(RAR_NOCRYPT)
+#if defined(RAR_NOCRYPT)
     return 0;
 #else
     RequestArcPassword();
@@ -771,7 +774,7 @@ size_t Archive::ReadHeader50()
         hd->MaxSize=Max(hd->PackSize,hd->UnpSize);
         hd->FileAttr=(uint)Raw.GetV();
         if ((hd->FileFlags & FHFL_UTIME)!=0)
-          hd->mtime=(time_t)Raw.Get4();
+          hd->mtime.SetUnix((time_t)Raw.Get4());
 
         hd->FileHash.Type=HASH_NONE;
         if ((hd->FileFlags & FHFL_CRC32)!=0)
@@ -784,7 +787,12 @@ size_t Archive::ReadHeader50()
 
         uint CompInfo=(uint)Raw.GetV();
         hd->Method=(CompInfo>>7) & 7;
-        hd->UnpVer=CompInfo & 0x3f;
+
+        // "+ 50" to not mix with old RAR format algorithms. For example,
+        // we may need to use the compression algorithm 15 in the future,
+        // but it was already used in RAR 1.5 and Unpack needs to distinguish
+        // them.
+        hd->UnpVer=(CompInfo & 0x3f) + 50;
 
         hd->HostOS=(byte)Raw.GetV();
         size_t NameSize=(size_t)Raw.GetV();
@@ -829,6 +837,20 @@ size_t Archive::ReadHeader50()
         if (!FileBlock && hd->CmpName(SUBHEAD_TYPE_CMT))
           MainComment=true;
 
+#if 0
+        // For RAR5 format we read the user specified recovery percent here.
+        // It would be useful to do it for shell extension too, so we display
+        // the correct recovery record size in archive properties. But then
+        // we would need to include the entire recovery record processing
+        // code to shell extension, which is not done now.
+        if (!FileBlock && hd->CmpName(SUBHEAD_TYPE_RR) && hd->SubData.Size()>0)
+        {
+          RecoveryPercent=hd->SubData[0];
+          RSBlockHeader Header;
+          GetRRInfo(this,&Header);
+          RecoverySize=Header.RecSectionSize*Header.RecCount;
+        }
+#endif
           
         if (BadCRC) // Add the file name to broken header message displayed above.
           uiMsg(UIERROR_FHEADERBROKEN,Archive::FileName,hd->FileName);
@@ -855,7 +877,7 @@ size_t Archive::ReadHeader50()
 }
 
 
-#if !defined(SHELL_EXT) && !defined(RAR_NOCRYPT)
+#if !defined(RAR_NOCRYPT)
 void Archive::RequestArcPassword()
 {
   if (!Cmd->Password.IsSet())
@@ -1006,25 +1028,35 @@ void Archive::ProcessExtra50(RawRead *Raw,size_t ExtraSize,BaseBlock *bb)
           }
           break;
         case FHEXTRA_HTIME:
-          if (FieldSize>=9)
+          if (FieldSize>=5)
           {
             byte Flags=(byte)Raw->GetV();
             bool UnixTime=(Flags & FHEXTRA_HTIME_UNIXTIME)!=0;
             if ((Flags & FHEXTRA_HTIME_MTIME)!=0)
               if (UnixTime)
-                hd->mtime=(time_t)Raw->Get4();
+                hd->mtime.SetUnix(Raw->Get4());
               else
-                hd->mtime.SetRaw(Raw->Get8());
+                hd->mtime.SetWin(Raw->Get8());
             if ((Flags & FHEXTRA_HTIME_CTIME)!=0)
               if (UnixTime)
-                hd->ctime=(time_t)Raw->Get4();
+                hd->ctime.SetUnix(Raw->Get4());
               else
-                hd->ctime.SetRaw(Raw->Get8());
+                hd->ctime.SetWin(Raw->Get8());
             if ((Flags & FHEXTRA_HTIME_ATIME)!=0)
               if (UnixTime)
-                hd->atime=(time_t)Raw->Get4();
+                hd->atime.SetUnix((time_t)Raw->Get4());
               else
-                hd->atime.SetRaw(Raw->Get8());
+                hd->atime.SetWin(Raw->Get8());
+            if (UnixTime && (Flags & FHEXTRA_HTIME_UNIX_NS)!=0) // Add nanoseconds.
+            {
+              uint ns;
+              if ((Flags & FHEXTRA_HTIME_MTIME)!=0 && (ns=(Raw->Get4() & 0x3fffffff))<1000000000)
+                hd->mtime.Adjust(ns);
+              if ((Flags & FHEXTRA_HTIME_CTIME)!=0 && (ns=(Raw->Get4() & 0x3fffffff))<1000000000)
+                hd->ctime.Adjust(ns);
+              if ((Flags & FHEXTRA_HTIME_ATIME)!=0 && (ns=(Raw->Get4() & 0x3fffffff))<1000000000)
+                hd->atime.Adjust(ns);
+            }
           }
           break;
         case FHEXTRA_VERSION:
@@ -1337,7 +1369,6 @@ int64 Archive::GetStartPos()
 }
 
 
-#ifndef SHELL_EXT
 bool Archive::ReadSubData(Array<byte> *UnpData,File *DestFile)
 {
   if (BrokenHeader)
@@ -1404,4 +1435,3 @@ bool Archive::ReadSubData(Array<byte> *UnpData,File *DestFile)
   }
   return true;
 }
-#endif
