@@ -1,8 +1,5 @@
 extern uint CRCTab[256];
 
-#define  rol(x,n,xsize)  (((x)<<(n)) | ((x)>>(xsize-(n))))
-#define  ror(x,n,xsize)  (((x)>>(n)) | ((x)<<(xsize-(n))))
-
 void CryptData::SetKey13(const char *Password)
 {
   Key13[0]=Key13[1]=Key13[2]=0;
@@ -12,7 +9,7 @@ void CryptData::SetKey13(const char *Password)
     Key13[0]+=P;
     Key13[1]^=P;
     Key13[2]+=P;
-    Key13[2]=(byte)rol(Key13[2],1,8);
+    Key13[2]=(byte)rotls(Key13[2],1,8);
   }
 }
 
@@ -73,8 +70,8 @@ void CryptData::Crypt15(byte *Data,size_t Count)
     Key15[1]^=CRCTab[(Key15[0] & 0x1fe)>>1];
     Key15[2]-=CRCTab[(Key15[0] & 0x1fe)>>1]>>16;
     Key15[0]^=Key15[2];
-    Key15[3]=ror(Key15[3]&0xffff,1,16)^Key15[1];
-    Key15[3]=ror(Key15[3]&0xffff,1,16);
+    Key15[3]=rotrs(Key15[3]&0xffff,1,16)^Key15[1];
+    Key15[3]=rotrs(Key15[3]&0xffff,1,16);
     Key15[0]^=Key15[3];
     *Data^=(byte)(Key15[0]>>8);
     Data++;
