@@ -6,9 +6,12 @@
 #define   MRetryAbort        L"_Retry_Abort"
 #define   MCopyright         L"\nRAR %s   Copyright (c) 1993-%d Alexander Roshal   %d %s %d"
 #define   MRegTo             L"\nRegistered to %s\n"
-#define   MShare             L"\nTrial version             Type RAR -? for help\n"
+#define   MShare             L"\nTrial version             Type 'rar -?' for help\n"
+#define   MRegKeyWarning     L"\nAvailable license key is valid only for %s\n"
 #define   MUCopyright        L"\nUNRAR %s freeware      Copyright (c) 1993-%d Alexander Roshal\n"
 #define   MBeta              L"beta"
+#define   Mx86               L"x86"
+#define   Mx64               L"x64"
 #define   MMonthJan          L"Jan"
 #define   MMonthFeb          L"Feb"
 #define   MMonthMar          L"Mar"
@@ -83,6 +86,7 @@
 #define   MCHelpSwINUL       L"\n  inul          Disable all messages"
 #define   MCHelpSwIOFF       L"\n  ioff          Turn PC off after completing an operation"
 #define   MCHelpSwISND       L"\n  isnd          Enable sound"
+#define   MCHelpSwIVER       L"\n  iver          Display the version number"
 #define   MCHelpSwK          L"\n  k             Lock archive"
 #define   MCHelpSwKB         L"\n  kb            Keep broken extracted files"
 #define   MCHelpSwLog        L"\n  log[f][=name] Write names to log file"
@@ -100,6 +104,7 @@
 #define   MCHelpSwOH         L"\n  oh            Save hard links as the link instead of the file"
 #define   MCHelpSwOI         L"\n  oi[0-4][:min] Save identical files as references"
 #define   MCHelpSwOL         L"\n  ol[a]         Process symbolic links as the link [absolute paths]"
+#define   MCHelpSwONI        L"\n  oni           Allow potentially incompatible names"
 #define   MCHelpSwOR         L"\n  or            Rename files automatically"
 #define   MCHelpSwOS         L"\n  os            Save NTFS streams"
 #define   MCHelpSwOW         L"\n  ow            Save or restore file owner and group"
@@ -126,7 +131,7 @@
 #define   MCHelpSwTO         L"\n  to<time>      Process files older than <time>"
 #define   MCHelpSwTA         L"\n  ta<date>      Process files modified after <date> in YYYYMMDDHHMMSS format"
 #define   MCHelpSwTB         L"\n  tb<date>      Process files modified before <date> in YYYYMMDDHHMMSS format"
-#define   MCHelpSwTS         L"\n  ts<m,c,a>[N]  Save or restore file time (modification, creation, access)"
+#define   MCHelpSwTS         L"\n  ts[m|c|a]     Save or restore file time (modification, creation, access)"
 #define   MCHelpSwU          L"\n  u             Update files"
 #define   MCHelpSwV          L"\n  v             Create volumes with size autodetection or list all volumes"
 #define   MCHelpSwVUnr       L"\n  v             List all volumes"
@@ -273,9 +278,7 @@
 #define   MListHostOS        L"Host OS"
 #define   MListFileVer       L"File version"
 #define   MListService       L"Service"
-#define   MListEAHead        L"\n   OS/2 extended attributes"
 #define   MListUOHead        L"\n   Unix Owner/Group data:    %-14s %-14s"
-#define   MListBeEAHead      L"\n   BeOS extended attributes"
 #define   MListNTACLHead     L"\n   NTFS security data"
 #define   MListStrmHead      L"\n   NTFS stream: %s"
 #define   MListUnkHead       L"\n   Unknown subheader type: 0x%04x"
@@ -318,14 +321,7 @@
 #define   MStreamBroken      L"\nERROR: %s stream data are corrupt\n"
 #define   MStreamUnknown     L"\nWARNING: Unknown format of %s stream data\n"
 #define   MInvalidName       L"\nERROR: Invalid file name %s"
-#define   MEABroken          L"\nERROR: %s extended attributes are corrupt\n"
-#define   MEAUnknHeader      L"\nWARNING: %s - unknown format of extended attributes\n"
-#define   MCannotSetEA       L"\nWARNING: cannot set extended attributes to %s\n"
-#define   MCannotGetEA       L"\nERROR: Cannot get extended attributes of %s\n"
-#define   MShowEA            L" (+EA)"
-#define   MSkipEA            L"\n...skipping extended attributes"
 #define   MProcessArc        L"\n\nProcessing archive %s"
-#define   MSyncScanError     L"\nFile search errors, cannot synchronize archive"
 #define   MCorrectingName    L"\nWARNING: Attempting to correct the invalid file name"
 #define   MUnpCannotMerge    L"\nWARNING: You need to start extraction from a previous volume to unpack %s"
 #define   MUnknownOption     L"\nERROR: Unknown option: %s"
@@ -347,7 +343,7 @@
 #define   MFAT32Size         L"\nWARNING: FAT32 file system does not support 4 GB or larger files"
 #define   MErrChangeAttr     L"\nWARNING: Cannot change attributes of %s"
 #define   MWrongSFXVer       L"\nERROR: default SFX module does not support RAR %d.%d archives"
-#define   MCannotEncName     L"\nCannot encrypt archive already contained encrypted files"
+#define   MCannotEncName     L"\nCannot encrypt archive already containing encrypted files"
 #define   MCannotEmail       L"\nCannot email the file %s"
 #define   MCopyrightS        L"\nRAR SFX archive" 
 #define   MSHelpCmd          L"\n\n<Commands>" 
@@ -374,6 +370,7 @@
 #define   MSearchDupFiles    L"\nSearching for identical files"
 #define   MNumFound          L"%d found."
 #define   MUnknownExtra      L"\nUnknown extra field in %s."
+#define   MCorruptExtra      L"\nCorrupt %s extra field in %s."
 #define   MCopyError         L"\nCannot copy %s to %s."
 #define   MCopyErrorHint     L"\nYou need to unpack the entire archive to create file reference entries."
 #define   MCopyingData       L"\nCopying data"
