@@ -594,7 +594,7 @@ int ParseVersionFileName(wchar *Name,bool Truncate)
 }
 
 
-#if !defined(SFX_MODULE) && !defined(SETUP)
+#if !defined(SFX_MODULE)
 // Get the name of first volume. Return the leftmost digit of volume number.
 wchar* VolNameToFirstName(const wchar *VolName,wchar *FirstName,size_t MaxSize,bool NewNumbering)
 {
@@ -733,16 +733,16 @@ static void GenArcName(wchar *ArcName,const wchar *GenerateMask,uint ArcNumber,b
 
   char Field[10][6];
 
-  sprintf(Field[0],"%04d",rlt.Year);
-  sprintf(Field[1],"%02d",rlt.Month);
-  sprintf(Field[2],"%02d",rlt.Day);
-  sprintf(Field[3],"%02d",rlt.Hour);
-  sprintf(Field[4],"%02d",rlt.Minute);
-  sprintf(Field[5],"%02d",rlt.Second);
-  sprintf(Field[6],"%02d",CurWeek);
-  sprintf(Field[7],"%d",WeekDay+1);
-  sprintf(Field[8],"%03d",rlt.yDay+1);
-  sprintf(Field[9],"%05d",ArcNumber);
+  sprintf(Field[0],"%04u",rlt.Year);
+  sprintf(Field[1],"%02u",rlt.Month);
+  sprintf(Field[2],"%02u",rlt.Day);
+  sprintf(Field[3],"%02u",rlt.Hour);
+  sprintf(Field[4],"%02u",rlt.Minute);
+  sprintf(Field[5],"%02u",rlt.Second);
+  sprintf(Field[6],"%02u",(uint)CurWeek);
+  sprintf(Field[7],"%u",(uint)WeekDay+1);
+  sprintf(Field[8],"%03u",rlt.yDay+1);
+  sprintf(Field[9],"%05u",ArcNumber);
 
   const wchar *MaskChars=L"YMDHISWAEN";
 
@@ -873,7 +873,7 @@ wchar* GetWideName(const char *Name,const wchar *NameW,wchar *DestW,size_t DestS
   if (DestSize>0)
     DestW[DestSize-1]=0;
 
-  return(DestW);
+  return DestW;
 }
 
 
@@ -958,4 +958,26 @@ void ConvertToPrecomposed(wchar *Name,size_t NameSize)
     wcsncpyz(Name,FileName,NameSize);
   }
 }
+
+
+// Remove trailing spaces and dots in file name and in dir names in path.
+void MakeNameCompatible(wchar *Name)
+{
+  int Src=0,Dest=0;
+  while (true)
+  {
+    if (IsPathDiv(Name[Src]) || Name[Src]==0)
+      for (int I=Dest-1;I>0 && (Name[I]==' ' || Name[I]=='.');I--)
+      {
+        if (IsPathDiv(Name[I-1])) // Permit path1/./path2 paths.
+          break;
+        Dest--;
+      }
+    Name[Dest]=Name[Src];
+    if (Name[Src]==0)
+      break;
+    Src++;
+    Dest++;
+  }
+}
 #endif
