@@ -2,6 +2,10 @@
 
 ComprDataIO::ComprDataIO()
 {
+#ifndef RAR_NOCRYPT
+  Crypt=new CryptData;
+  Decrypt=new CryptData;
+#endif
 
   Init();
 }
@@ -33,6 +37,13 @@ void ComprDataIO::Init()
 }
 
 
+ComprDataIO::~ComprDataIO()
+{
+#ifndef RAR_NOCRYPT
+  delete Crypt;
+  delete Decrypt;
+#endif
+}
 
 
 
@@ -127,7 +138,7 @@ int ComprDataIO::UnpRead(byte *Addr,size_t Count)
     ReadSize=TotalRead;
 #ifndef RAR_NOCRYPT
     if (Decryption)
-      Decrypt.DecryptBlock(Addr,ReadSize);
+      Decrypt->DecryptBlock(Addr,ReadSize);
 #endif
   }
   Wait();
@@ -273,9 +284,9 @@ void ComprDataIO::SetEncryption(bool Encrypt,CRYPT_METHOD Method,
 {
 #ifndef RAR_NOCRYPT
   if (Encrypt)
-    Encryption=Crypt.SetCryptKeys(true,Method,Password,Salt,InitV,Lg2Cnt,HashKey,PswCheck);
+    Encryption=Crypt->SetCryptKeys(true,Method,Password,Salt,InitV,Lg2Cnt,HashKey,PswCheck);
   else
-    Decryption=Decrypt.SetCryptKeys(false,Method,Password,Salt,InitV,Lg2Cnt,HashKey,PswCheck);
+    Decryption=Decrypt->SetCryptKeys(false,Method,Password,Salt,InitV,Lg2Cnt,HashKey,PswCheck);
 #endif
 }
 
@@ -284,7 +295,7 @@ void ComprDataIO::SetEncryption(bool Encrypt,CRYPT_METHOD Method,
 void ComprDataIO::SetAV15Encryption()
 {
   Decryption=true;
-  Decrypt.SetAV15Encryption();
+  Decrypt->SetAV15Encryption();
 }
 #endif
 
@@ -293,7 +304,7 @@ void ComprDataIO::SetAV15Encryption()
 void ComprDataIO::SetCmt13Encryption()
 {
   Decryption=true;
-  Decrypt.SetCmt13Encryption();
+  Decrypt->SetCmt13Encryption();
 }
 #endif
 
