@@ -1,6 +1,6 @@
 #include "rar.hpp"
 
-#ifdef _WIN_ALL
+#if defined(_WIN_ALL)
 typedef BOOL (WINAPI *CRYPTPROTECTMEMORY)(LPVOID pData,DWORD cbData,DWORD dwFlags);
 typedef BOOL (WINAPI *CRYPTUNPROTECTMEMORY)(LPVOID pData,DWORD cbData,DWORD dwFlags);
 
@@ -38,6 +38,7 @@ class CryptLoader
         hCrypt = LoadSysLibrary(L"Crypt32.dll");
         if (hCrypt != NULL)
         {
+          // Available since Vista.
           pCryptProtectMemory = (CRYPTPROTECTMEMORY)GetProcAddress(hCrypt, "CryptProtectMemory");
           pCryptUnprotectMemory = (CRYPTUNPROTECTMEMORY)GetProcAddress(hCrypt, "CryptUnprotectMemory");
         }
@@ -78,6 +79,8 @@ void SecPassword::Clean()
 // So we use our own function for this purpose.
 void cleandata(void *data,size_t size)
 {
+  if (data==NULL || size==0)
+    return;
 #if defined(_WIN_ALL) && defined(_MSC_VER)
   SecureZeroMemory(data,size);
 #else
@@ -117,6 +120,8 @@ void SecPassword::Get(wchar *Psw,size_t MaxSize)
 }
 
 
+
+
 void SecPassword::Set(const wchar *Psw)
 {
   if (*Psw==0)
@@ -160,7 +165,9 @@ bool SecPassword::operator == (SecPassword &psw)
 
 void SecHideData(void *Data,size_t DataSize,bool Encode,bool CrossProcess)
 {
-#ifdef _WIN_ALL
+  // CryptProtectMemory is not available in UWP and CryptProtectData
+  // increases data size not allowing in place conversion.
+#if defined(_WIN_ALL)
   // Try to utilize the secure Crypt[Un]ProtectMemory if possible.
   if (GlobalCryptLoader.pCryptProtectMemory==NULL)
     GlobalCryptLoader.Load();
