@@ -24,10 +24,14 @@ void Wait();
 bool EmailFile(const wchar *FileName,const wchar *MailToW);
 void Shutdown();
 
+#ifdef _WIN_ALL
+HMODULE WINAPI LoadSysLibrary(const wchar *Name);
+bool IsUserAdmin();
+#endif
 
 
 #ifdef USE_SSE
-enum SSE_VERSION {SSE_NONE,SSE_SSE,SSE_SSE2,SSE_SSSE3,SSE_SSE41};
+enum SSE_VERSION {SSE_NONE,SSE_SSE,SSE_SSE2,SSE_SSSE3,SSE_SSE41,SSE_AVX2};
 SSE_VERSION GetSSEVersion();
 extern SSE_VERSION _SSE_Version;
 #endif
