@@ -4,17 +4,31 @@
 // Maximum allowed number of compressed bits processed in quick mode.
 #define MAX_QUICK_DECODE_BITS      10
 
-// Maximum number of filters per entire data block.
+// Maximum number of filters per entire data block. Must be at least
+// twice more than MAX_PACK_FILTERS to store filters from two data blocks.
 #define MAX_UNPACK_FILTERS       8192
 
 // Maximum number of filters per entire data block for RAR3 unpack.
-#define MAX3_FILTERS             1024
+// Must be at least twice more than v3_MAX_PACK_FILTERS to store filters
+// from two data blocks.
+#define MAX3_UNPACK_FILTERS      8192
 
-// Write data in 4 MB or smaller blocks.
-#define UNPACK_MAX_WRITE     0x400000
+// Limit maximum number of channels in RAR3 delta filter to some reasonable
+// value to prevent too slow processing of corrupt archives with invalid
+// channels number. Must be equal or larger than v3_MAX_FILTER_CHANNELS.
+// No need to provide it for RAR5, which uses only 5 bits to store channels.
+#define MAX3_UNPACK_CHANNELS      1024
+
+// Maximum size of single filter block. We restrict it to limit memory
+// allocation. Must be equal or larger than MAX_ANALYZE_SIZE.
+#define MAX_FILTER_BLOCK_SIZE 0x400000
+
+// Write data in 4 MB or smaller blocks. Must not exceed PACK_MAX_WRITE,
+// so we keep number of buffered filter in unpacker reasonable.
+#define UNPACK_MAX_WRITE      0x400000
 
 // Decode compressed bit fields to alphabet numbers.
-struct DecodeTable
+struct DecodeTable:PackDef
 {
   // Real size of DecodeNum table.
   uint MaxNum;
@@ -133,8 +147,8 @@ struct UnpackFilter
   uint BlockStart;
   uint BlockLength;
   byte Channels;
-  uint Width;
-  byte PosR;
+//  uint Width;
+//  byte PosR;
   bool NextWindow;
 };
 
@@ -143,10 +157,9 @@ struct UnpackFilter30
 {
   unsigned int BlockStart;
   unsigned int BlockLength;
-  unsigned int ExecCount;
   bool NextWindow;
 
-  // position of parent filter in Filters array used as prototype for filter
+  // Position of parent filter in Filters array used as prototype for filter
   // in PrgStack array. Not defined for filters in Filters array.
   unsigned int ParentFilter;
 
@@ -171,6 +184,8 @@ class FragmentedWindow
 {
   private:
     enum {MAX_MEM_BLOCKS=32};
+
+    void Reset();
     byte *Mem[MAX_MEM_BLOCKS];
     size_t MemSize[MAX_MEM_BLOCKS];
   public:
@@ -184,7 +199,7 @@ class FragmentedWindow
 };
 
 
-class Unpack
+class Unpack:PackDef
 {
   private:
 
@@ -192,12 +207,11 @@ class Unpack
     void Unpack5MT(bool Solid);
     bool UnpReadBuf();
     void UnpWriteBuf();
-    uint FilterItanium_GetBits(byte *Data,int BitPos,int BitCount);
-    void FilterItanium_SetBits(byte *Data,uint BitField,int BitPos,int BitCount);
     byte* ApplyFilter(byte *Data,uint DataSize,UnpackFilter *Flt);
     void UnpWriteArea(size_t StartPtr,size_t EndPtr);
     void UnpWriteData(byte *Data,size_t Size);
     _forceinline uint SlotToLength(BitInput &Inp,uint Slot);
+    void UnpInitData50(bool Solid);
     bool ReadBlockHeader(BitInput &Inp,UnpackBlockHeader &Header);
     bool ReadTables(BitInput &Inp,UnpackBlockHeader &Header,UnpackBlockTables &Tables);
     void MakeDecodeTables(byte *LengthTable,DecodeTable *Dec,uint Size);
@@ -294,7 +308,9 @@ class Unpack
     DecodeTable MD[4]; // Decode multimedia data, up to 4 channels.
 
     unsigned char UnpOldTable20[MC20*4];
-    int UnpAudioBlock,UnpChannels,UnpCurChannel,UnpChannelDelta;
+    bool UnpAudioBlock;
+    uint UnpChannels,UnpCurChannel;
+    int UnpChannelDelta;
     void CopyString20(uint Length,uint Distance);
     bool ReadTables20();
     void UnpWriteBuf20();
@@ -309,7 +325,7 @@ class Unpack
 
     void UnpInitData30(bool Solid);
     void Unpack29(bool Solid);
-    void InitFilters30();
+    void InitFilters30(bool Solid);
     bool ReadEndOfBlock();
     bool ReadVMCode();
     bool ReadVMCodePPM();
@@ -325,10 +341,15 @@ class Unpack
     ModelPPM PPM;
     int PPMEscChar;
 
-    byte UnpOldTable[HUFF_TABLE_SIZE];
+    byte UnpOldTable[HUFF_TABLE_SIZE30];
     int UnpBlockType;
 
-    bool TablesRead;
+    // If we already read decoding tables for Unpack v2,v3,v5.
+    // We should not use a single variable for all algorithm versions,
+    // because we can have a corrupt archive with one algorithm file
+    // followed by another algorithm file with "solid" flag and we do not
+    // want to reuse tables from one algorithm in another.
+    bool TablesRead2,TablesRead3,TablesRead5;
 
     // Virtual machine to execute filters code.
     RarVM VM;
@@ -355,7 +376,7 @@ class Unpack
     Unpack(ComprDataIO *DataIO);
     ~Unpack();
     void Init(size_t WinSize,bool Solid);
-    void DoUnpack(int Method,bool Solid);
+    void DoUnpack(uint Method,bool Solid);
     bool IsFileExtracted() {return(FileExtracted);}
     void SetDestSize(int64 DestSize) {DestUnpSize=DestSize;FileExtracted=false;}
     void SetSuspended(bool Suspended) {Unpack::Suspended=Suspended;}
