@@ -42,7 +42,7 @@ void Unpack::Unpack29(bool Solid)
     UnpInitData(Solid);
     if (!UnpReadBuf30())
       return;
-    if ((!Solid || !TablesRead) && !ReadTables30())
+    if ((!Solid || !TablesRead3) && !ReadTables30())
       return;
   }
 
@@ -133,7 +133,7 @@ void Unpack::Unpack29(bool Solid)
       continue;
     }
 
-    int Number=DecodeNumber(Inp,&BlockTables.LD);
+    uint Number=DecodeNumber(Inp,&BlockTables.LD);
     if (Number<256)
     {
       Window[UnpPtr++]=(byte)Number;
@@ -141,15 +141,15 @@ void Unpack::Unpack29(bool Solid)
     }
     if (Number>=271)
     {
-      int Length=LDecode[Number-=271]+3;
+      uint Length=LDecode[Number-=271]+3;
       if ((Bits=LBits[Number])>0)
       {
         Length+=Inp.getbits()>>(16-Bits);
         Inp.addbits(Bits);
       }
 
-      int DistNumber=DecodeNumber(Inp,&BlockTables.DD);
-      unsigned int Distance=DDecode[DistNumber]+1;
+      uint DistNumber=DecodeNumber(Inp,&BlockTables.DD);
+      uint Distance=DDecode[DistNumber]+1;
       if ((Bits=DBits[DistNumber])>0)
       {
         if (DistNumber>9)
@@ -166,7 +166,7 @@ void Unpack::Unpack29(bool Solid)
           }
           else
           {
-            int LowDist=DecodeNumber(Inp,&BlockTables.LDD);
+            uint LowDist=DecodeNumber(Inp,&BlockTables.LDD);
             if (LowDist==16)
             {
               LowDistRepCount=LOW_DIST_REP_COUNT-1;
@@ -189,7 +189,7 @@ void Unpack::Unpack29(bool Solid)
       if (Distance>=0x2000)
       {
         Length++;
-        if (Distance>=0x40000L)
+        if (Distance>=0x40000)
           Length++;
       }
 
@@ -218,13 +218,13 @@ void Unpack::Unpack29(bool Solid)
     }
     if (Number<263)
     {
-      int DistNum=Number-259;
-      unsigned int Distance=OldDist[DistNum];
-      for (int I=DistNum;I>0;I--)
+      uint DistNum=Number-259;
+      uint Distance=OldDist[DistNum];
+      for (uint I=DistNum;I>0;I--)
         OldDist[I]=OldDist[I-1];
       OldDist[0]=Distance;
 
-      int LengthNumber=DecodeNumber(Inp,&BlockTables.RD);
+      uint LengthNumber=DecodeNumber(Inp,&BlockTables.RD);
       int Length=LDecode[LengthNumber]+2;
       if ((Bits=LBits[LengthNumber])>0)
       {
@@ -237,7 +237,7 @@ void Unpack::Unpack29(bool Solid)
     }
     if (Number<272)
     {
-      unsigned int Distance=SDDecode[Number-=263]+1;
+      uint Distance=SDDecode[Number-=263]+1;
       if ((Bits=SDBits[Number])>0)
       {
         Distance+=Inp.getbits()>>(16-Bits);
@@ -274,11 +274,11 @@ bool Unpack::ReadEndOfBlock()
     NewTable=(BitField & 0x4000)!=0;
     Inp.addbits(2);
   }
-  TablesRead=!NewTable;
+  TablesRead3=!NewTable;
 
   // Quit immediately if "new file" flag is set. If "new table" flag
   // is present, we'll read the table in beginning of next file
-  // based on 'TablesRead' 'false' value.
+  // based on 'TablesRead3' 'false' value.
   if (NewFile)
     return false;
   return ReadTables30(); // Quit only if we failed to read tables.
@@ -290,9 +290,9 @@ bool Unpack::ReadVMCode()
   // Entire VM code is guaranteed to fully present in block defined 
   // by current Huffman table. Compressor checks that VM code does not cross
   // Huffman block boundaries.
-  unsigned int FirstByte=Inp.getbits()>>8;
+  uint FirstByte=Inp.getbits()>>8;
   Inp.addbits(8);
-  int Length=(FirstByte & 7)+1;
+  uint Length=(FirstByte & 7)+1;
   if (Length==7)
   {
     Length=(Inp.getbits()>>8)+7;
@@ -304,8 +304,10 @@ bool Unpack::ReadVMCode()
       Length=Inp.getbits();
       Inp.addbits(16);
     }
+  if (Length==0)
+    return false;
   Array<byte> VMCode(Length);
-  for (int I=0;I<Length;I++)
+  for (uint I=0;I<Length;I++)
   {
     // Try to read the new buffer if only one byte is left.
     // But if we read all bytes except the last, one byte is enough.
@@ -320,15 +322,15 @@ bool Unpack::ReadVMCode()
 
 bool Unpack::ReadVMCodePPM()
 {
-  unsigned int FirstByte=SafePPMDecodeChar();
+  uint FirstByte=SafePPMDecodeChar();
   if ((int)FirstByte==-1)
     return false;
-  int Length=(FirstByte & 7)+1;
+  uint Length=(FirstByte & 7)+1;
   if (Length==7)
   {
     int B1=SafePPMDecodeChar();
     if (B1==-1)
-      return(false);
+      return false;
     Length=B1+7;
   }
   else
@@ -342,12 +344,14 @@ bool Unpack::ReadVMCodePPM()
         return false;
       Length=B1*256+B2;
     }
+  if (Length==0)
+    return false;
   Array<byte> VMCode(Length);
-  for (int I=0;I<Length;I++)
+  for (uint I=0;I<Length;I++)
   {
     int Ch=SafePPMDecodeChar();
     if (Ch==-1)
-      return(false);
+      return false;
     VMCode[I]=Ch;
   }
   return AddVMCode(FirstByte,&VMCode[0],Length);
@@ -393,21 +397,19 @@ bool Unpack::AddVMCode(uint FirstByte,byte *Code,int CodeSize)
     Filters30[Filters30.Size()-1]=Filter=new UnpackFilter30;
     StackFilter->ParentFilter=(uint)(Filters30.Size()-1);
 
-    // Reserve one item, where we store the data block length of our new
-    // filter entry. We'll set it to real block length below, after reading
-    // it. But we need to initialize it now, because when processing corrupt
+    // Reserve one item to store the data block length of our new filter 
+    // entry. We'll set it to real block length below, after reading it.
+    // But we need to initialize it now, because when processing corrupt
     // data, we can access this item even before we set it to real value.
     OldFilterLengths.Push(0);
-    Filter->ExecCount=0;
   }
   else  // Filter was used in the past.
   {
     Filter=Filters30[FiltPos];
     StackFilter->ParentFilter=FiltPos;
-    Filter->ExecCount++;
   }
 
-  int EmptyCount=0;
+  uint EmptyCount=0;
   for (uint I=0;I<PrgStack.Size();I++)
   {
     PrgStack[I-EmptyCount]=PrgStack[I];
@@ -426,9 +428,8 @@ bool Unpack::AddVMCode(uint FirstByte,byte *Code,int CodeSize)
     PrgStack.Add(1);
     EmptyCount=1;
   }
-  int StackPos=(int)(PrgStack.Size()-EmptyCount);
+  size_t StackPos=PrgStack.Size()-EmptyCount;
   PrgStack[StackPos]=StackFilter;
-  StackFilter->ExecCount=Filter->ExecCount;
  
   uint BlockStart=RarVM::ReadData(VMCodeInp);
   if ((FirstByte & 0x40)!=0)
@@ -444,9 +445,9 @@ bool Unpack::AddVMCode(uint FirstByte,byte *Code,int CodeSize)
   else
   {
     // Set the data block size to same value as the previous block size
-    // for same filter. It is possible on corrupt data to access here a new 
-    // and not filled yet item of OldFilterLengths array. This is why above
-    // we set new OldFilterLengths items to zero.
+    // for same filter. It is possible for corrupt data to access a new 
+    // and not filled yet item of OldFilterLengths array here. This is why
+    // we set new OldFilterLengths items to zero above.
     StackFilter->BlockLength=FiltPos<OldFilterLengths.Size() ? OldFilterLengths[FiltPos]:0;
   }
 
@@ -455,15 +456,13 @@ bool Unpack::AddVMCode(uint FirstByte,byte *Code,int CodeSize)
 //  DebugLog("\nNextWindow: UnpPtr=%08x WrPtr=%08x BlockStart=%08x",UnpPtr,WrPtr,BlockStart);
 
   memset(StackFilter->Prg.InitR,0,sizeof(StackFilter->Prg.InitR));
-  StackFilter->Prg.InitR[3]=VM_GLOBALMEMADDR;
   StackFilter->Prg.InitR[4]=StackFilter->BlockLength;
-  StackFilter->Prg.InitR[5]=StackFilter->ExecCount;
 
   if ((FirstByte & 0x10)!=0) // Set registers to optional parameters if any.
   {
     uint InitMask=VMCodeInp.fgetbits()>>9;
     VMCodeInp.faddbits(7);
-    for (int I=0;I<7;I++)
+    for (uint I=0;I<7;I++)
       if (InitMask & (1<<I))
         StackFilter->Prg.InitR[I]=RarVM::ReadData(VMCodeInp);
   }
@@ -483,49 +482,8 @@ bool Unpack::AddVMCode(uint FirstByte,byte *Code,int CodeSize)
     }
     VM.Prepare(&VMCode[0],VMCodeSize,&Filter->Prg);
   }
-  StackFilter->Prg.AltCmd=&Filter->Prg.Cmd[0];
-  StackFilter->Prg.CmdCount=Filter->Prg.CmdCount;
+  StackFilter->Prg.Type=Filter->Prg.Type;
 
-  size_t StaticDataSize=Filter->Prg.StaticData.Size();
-  if (StaticDataSize>0 && StaticDataSize<VM_GLOBALMEMSIZE)
-  {
-    // read statically defined data contained in DB commands
-    StackFilter->Prg.StaticData.Add(StaticDataSize);
-    memcpy(&StackFilter->Prg.StaticData[0],&Filter->Prg.StaticData[0],StaticDataSize);
-  }
-
-  if (StackFilter->Prg.GlobalData.Size()<VM_FIXEDGLOBALSIZE)
-  {
-    StackFilter->Prg.GlobalData.Reset();
-    StackFilter->Prg.GlobalData.Add(VM_FIXEDGLOBALSIZE);
-  }
-  byte *GlobalData=&StackFilter->Prg.GlobalData[0];
-  for (int I=0;I<7;I++)
-    VM.SetLowEndianValue((uint *)&GlobalData[I*4],StackFilter->Prg.InitR[I]);
-  VM.SetLowEndianValue((uint *)&GlobalData[0x1c],StackFilter->BlockLength);
-  VM.SetLowEndianValue((uint *)&GlobalData[0x20],0);
-  VM.SetLowEndianValue((uint *)&GlobalData[0x2c],StackFilter->ExecCount);
-  memset(&GlobalData[0x30],0,16);
-
-  if ((FirstByte & 8)!=0) // Put the data block passed as parameter if any.
-  {
-    if (VMCodeInp.Overflow(3))
-      return false;
-    uint DataSize=RarVM::ReadData(VMCodeInp);
-    if (DataSize>VM_GLOBALMEMSIZE-VM_FIXEDGLOBALSIZE)
-      return false;
-    size_t CurSize=StackFilter->Prg.GlobalData.Size();
-    if (CurSize<DataSize+VM_FIXEDGLOBALSIZE)
-      StackFilter->Prg.GlobalData.Add(DataSize+VM_FIXEDGLOBALSIZE-CurSize);
-    byte *GlobalData=&StackFilter->Prg.GlobalData[VM_FIXEDGLOBALSIZE];
-    for (uint I=0;I<DataSize;I++)
-    {
-      if (VMCodeInp.Overflow(3))
-        return false;
-      GlobalData[I]=VMCodeInp.fgetbits()>>8;
-      VMCodeInp.faddbits(8);
-    }
-  }
   return true;
 }
 
@@ -603,25 +561,8 @@ void Unpack::UnpWriteBuf30()
         VM_PreparedProgram *ParentPrg=&Filters30[flt->ParentFilter]->Prg;
         VM_PreparedProgram *Prg=&flt->Prg;
 
-        if (ParentPrg->GlobalData.Size()>VM_FIXEDGLOBALSIZE)
-        {
-          // Copy global data from previous script execution if any.
-          Prg->GlobalData.Alloc(ParentPrg->GlobalData.Size());
-          memcpy(&Prg->GlobalData[VM_FIXEDGLOBALSIZE],&ParentPrg->GlobalData[VM_FIXEDGLOBALSIZE],ParentPrg->GlobalData.Size()-VM_FIXEDGLOBALSIZE);
-        }
-
         ExecuteCode(Prg);
 
-        if (Prg->GlobalData.Size()>VM_FIXEDGLOBALSIZE)
-        {
-          // Save global data for next script execution.
-          if (ParentPrg->GlobalData.Size()<Prg->GlobalData.Size())
-            ParentPrg->GlobalData.Alloc(Prg->GlobalData.Size());
-          memcpy(&ParentPrg->GlobalData[VM_FIXEDGLOBALSIZE],&Prg->GlobalData[VM_FIXEDGLOBALSIZE],Prg->GlobalData.Size()-VM_FIXEDGLOBALSIZE);
-        }
-        else
-          ParentPrg->GlobalData.Reset();
-
         byte *FilteredData=Prg->FilteredData;
         unsigned int FilteredDataSize=Prg->FilteredDataSize;
 
@@ -642,25 +583,8 @@ void Unpack::UnpWriteBuf30()
           VM_PreparedProgram *ParentPrg=&Filters30[NextFilter->ParentFilter]->Prg;
           VM_PreparedProgram *NextPrg=&NextFilter->Prg;
 
-          if (ParentPrg->GlobalData.Size()>VM_FIXEDGLOBALSIZE)
-          {
-            // Copy global data from previous script execution if any.
-            NextPrg->GlobalData.Alloc(ParentPrg->GlobalData.Size());
-            memcpy(&NextPrg->GlobalData[VM_FIXEDGLOBALSIZE],&ParentPrg->GlobalData[VM_FIXEDGLOBALSIZE],ParentPrg->GlobalData.Size()-VM_FIXEDGLOBALSIZE);
-          }
-
           ExecuteCode(NextPrg);
 
-          if (NextPrg->GlobalData.Size()>VM_FIXEDGLOBALSIZE)
-          {
-            // Save global data for next script execution.
-            if (ParentPrg->GlobalData.Size()<NextPrg->GlobalData.Size())
-              ParentPrg->GlobalData.Alloc(NextPrg->GlobalData.Size());
-            memcpy(&ParentPrg->GlobalData[VM_FIXEDGLOBALSIZE],&NextPrg->GlobalData[VM_FIXEDGLOBALSIZE],NextPrg->GlobalData.Size()-VM_FIXEDGLOBALSIZE);
-          }
-          else
-            ParentPrg->GlobalData.Reset();
-
           FilteredData=NextPrg->FilteredData;
           FilteredDataSize=NextPrg->FilteredDataSize;
           I++;
@@ -696,13 +620,8 @@ void Unpack::UnpWriteBuf30()
 
 void Unpack::ExecuteCode(VM_PreparedProgram *Prg)
 {
-  if (Prg->GlobalData.Size()>0)
-  {
-    Prg->InitR[6]=(uint)WrittenFileSize;
-    VM.SetLowEndianValue((uint *)&Prg->GlobalData[0x24],(uint)WrittenFileSize);
-    VM.SetLowEndianValue((uint *)&Prg->GlobalData[0x28],(uint)(WrittenFileSize>>32));
-    VM.Execute(Prg);
-  }
+  Prg->InitR[6]=(uint)WrittenFileSize;
+  VM.Execute(Prg);
 }
 
 
@@ -729,13 +648,13 @@ bool Unpack::ReadTables30()
     memset(UnpOldTable,0,sizeof(UnpOldTable));
   Inp.faddbits(2);
 
-  for (int I=0;I<BC;I++)
+  for (uint I=0;I<BC;I++)
   {
-    int Length=(byte)(Inp.fgetbits() >> 12);
+    uint Length=(byte)(Inp.fgetbits() >> 12);
     Inp.faddbits(4);
     if (Length==15)
     {
-      int ZeroCount=(byte)(Inp.fgetbits() >> 12);
+      uint ZeroCount=(byte)(Inp.fgetbits() >> 12);
       Inp.faddbits(4);
       if (ZeroCount==0)
         BitLength[I]=15;
@@ -752,13 +671,13 @@ bool Unpack::ReadTables30()
   }
   MakeDecodeTables(BitLength,&BlockTables.BD,BC30);
 
-  const int TableSize=HUFF_TABLE_SIZE30;
-  for (int I=0;I<TableSize;)
+  const uint TableSize=HUFF_TABLE_SIZE30;
+  for (uint I=0;I<TableSize;)
   {
     if (Inp.InAddr>ReadTop-5)
       if (!UnpReadBuf30())
         return(false);
-    int Number=DecodeNumber(Inp,&BlockTables.BD);
+    uint Number=DecodeNumber(Inp,&BlockTables.BD);
     if (Number<16)
     {
       Table[I]=(Number+UnpOldTable[I]) & 0xf;
@@ -767,7 +686,7 @@ bool Unpack::ReadTables30()
     else
       if (Number<18)
       {
-        int N;
+        uint N;
         if (Number==16)
         {
           N=(Inp.fgetbits() >> 13)+3;
@@ -778,7 +697,9 @@ bool Unpack::ReadTables30()
           N=(Inp.fgetbits() >> 9)+11;
           Inp.faddbits(7);
         }
-        if (I>0)
+        if (I==0)
+          return false; // We cannot have "repeat previous" code at the first position.
+        else
           while (N-- > 0 && I<TableSize)
           {
             Table[I]=Table[I-1];
@@ -787,7 +708,7 @@ bool Unpack::ReadTables30()
       }
       else
       {
-        int N;
+        uint N;
         if (Number==18)
         {
           N=(Inp.fgetbits() >> 13)+3;
@@ -802,7 +723,7 @@ bool Unpack::ReadTables30()
           Table[I++]=0;
       }
   }
-  TablesRead=true;
+  TablesRead3=true;
   if (Inp.InAddr>ReadTop)
     return false;
   MakeDecodeTables(&Table[0],&BlockTables.LD,NC30);
@@ -818,7 +739,7 @@ void Unpack::UnpInitData30(bool Solid)
 {
   if (!Solid)
   {
-    TablesRead=false;
+    TablesRead3=false;
     memset(UnpOldTable,0,sizeof(UnpOldTable));
     PPMEscChar=2;
     UnpBlockType=BLOCK_LZ;
