@@ -43,6 +43,7 @@ void Unpack::InitMT()
       {
         // Typical number of items in RAR blocks does not exceed 0x4000.
         CurData->DecodedAllocated=0x4100;
+        // It will be freed in the object destructor, not in this file.
         CurData->Decoded=(UnpackDecodedItem *)malloc(CurData->DecodedAllocated*sizeof(UnpackDecodedItem));
         if (CurData->Decoded==NULL)
           ErrHandler.MemoryError();
@@ -96,7 +97,6 @@ void Unpack::Unpack5MT(bool Solid)
     if (ReadSize>0 && DataSize<TooSmallToProcess)
       continue;
 
-    bool BufferProcessed=false;
     while (BlockStart<DataSize && !Done)
     {
       uint BlockNumber=0,BlockNumberMT=0;
@@ -133,11 +133,13 @@ void Unpack::Unpack5MT(bool Solid)
         if (!CurData->HeaderRead)
         {
           CurData->HeaderRead=true;
-          if (!ReadBlockHeader(CurData->Inp,CurData->BlockHeader))
+          if (!ReadBlockHeader(CurData->Inp,CurData->BlockHeader) ||
+              !CurData->BlockHeader.TablePresent && !TablesRead5)
           {
             Done=true;
             break;
           }
+          TablesRead5=true;
         }
 
         // To prevent too high memory use we switch to single threaded mode
@@ -275,6 +277,7 @@ void Unpack::Unpack5MT(bool Solid)
       }
     }
   }
+  UnpPtr&=MaxWinMask; // ProcessDecoded and maybe others can leave UnpPtr > MaxWinMask here.
   UnpWriteBuf();
 
   BlockHeader=UnpThreadData[LastBlockNum].BlockHeader;
@@ -328,9 +331,10 @@ void Unpack::UnpackDecode(UnpackThreadData &D)
     if (D.DecodedSize>D.DecodedAllocated-8) // Filter can use several slots.
     {
       D.DecodedAllocated=D.DecodedAllocated*2;
-      D.Decoded=(UnpackDecodedItem *)realloc(D.Decoded,D.DecodedAllocated*sizeof(UnpackDecodedItem));
-      if (D.Decoded==NULL)
-        ErrHandler.MemoryError();
+      void *Decoded=realloc(D.Decoded,D.DecodedAllocated*sizeof(UnpackDecodedItem));
+      if (Decoded==NULL)
+        ErrHandler.MemoryError(); // D.Decoded will be freed in the destructor.
+      D.Decoded=(UnpackDecodedItem *)Decoded;
     }
 
     UnpackDecodedItem *CurItem=D.Decoded+D.DecodedSize++;
@@ -456,7 +460,7 @@ bool Unpack::ProcessDecoded(UnpackThreadData &D)
 
     if (Item->Type==UNPDT_LITERAL)
     {
-#if defined(LITTLE_ENDIAN) && defined(PRESENT_INT32) && defined(ALLOW_MISALIGNED)
+#if defined(LITTLE_ENDIAN) && defined(ALLOW_MISALIGNED)
       if (Item->Length==3 && UnpPtr<MaxWinSize-4)
       {
         *(uint32 *)(Window+UnpPtr)=*(uint32 *)Item->Literal;
